#pragma once
#include "stdafx.h"

namespace gc {

	using namespace System;
	using namespace System::IO;
	using namespace System::Windows::Forms;

		
	GraphDoc::GraphDoc(){
		pGraph = gcnew CGraph();
	}

	GraphDoc::~GraphDoc(){

	}


//  How many traces ?

	int GraphDoc::CountTraceNumberB(String^ filename)
	{
		int i,m,len;
		int nt;
		double a0,d0,d1;
		bool Success=false;
		FileStream^ fs;
        BinaryReader^ br;

		fs = gcnew FileStream(filename, FileMode::Open, FileAccess::Read);
		br = gcnew BinaryReader(fs);

		len = (int)br->BaseStream->Length/sizeof(double);

		for(m=2 ; m < len/2 ; m++){
			if(len % m ==0){
				br->BaseStream->Position=0;
				a0 = br->ReadDouble();
				br->BaseStream->Position=m*sizeof(double);
				d0 = br->ReadDouble() -a0;
				if(d0!=0){
					i=2;
					while(1){
						br->BaseStream->Position=i*m*sizeof(double);
						d1 = (br->ReadDouble()-a0)/(double)i;
						if(System::Math::Abs((d1-d0)/d0)>1e-3 ){
							break;
						}
						i++;
						if((i*m>=len) || (i>30)){
							Success=true;
							break;
						}
					}
				}
			}
			if(Success)break;
		}
		br->Close();
		fs->Close();

		nt = m-1;

		if(!Success){
			inputdialog^ idb = gcnew inputdialog("Input No. of traces:");
			idb->ShowDialog();
			if(idb->ResultOK){
				nt = idb->n;
			}else{
				return 0;
			}
		}
		return nt;
	}


	int GraphDoc::CountTraceNumberT(String^ filename)
	{
        StreamReader^ sr = gcnew StreamReader(filename);
		String^ s;

		s = sr->ReadLine();
		sr->Close();

		int i;
		int count = 0;
		bool bWhite=true;
		for(i=0;i<s->Length;i++){
			if(Char::IsWhiteSpace(s,i)){
				if(!bWhite){ // changed from non-white -> white
					bWhite=true;
				}
			}else{ // not while

				if(bWhite){ // changed from white -> non-white
					bWhite=false;
					count++;
				}
			}
		}
		count = count -1;
		if(count<0)count=0;
		return count;
	}

	int GraphDoc::ReadBinaryDataFile(String^ strFileName, int nt)
	{
		int i,j;
		int npts;
		interior_ptr<double> p;
		array<Byte>^ buf;
		FileStream^ fs;
        BinaryReader^ br;

		fs = gcnew FileStream(strFileName, FileMode::Open, FileAccess::Read);
		br = gcnew BinaryReader(fs);

		npts = (int)(br->BaseStream->Length)/((nt+1)*sizeof(double));

		pGraph->ntraces = nt;
		pGraph->nwaves = nt+1;
		pGraph->npts = npts;

		br->BaseStream->Position=0;

		pGraph->MakeWaves();

		buf = gcnew array<Byte>(pGraph->nwaves*sizeof(double));

		for(i=0;i<pGraph->npts;i++){

			buf = br->ReadBytes(pGraph->nwaves*sizeof(double));
			p = interior_ptr<double>(&buf[0]);
			for(j=0;j<pGraph->nwaves;j++){
				pGraph->W[j]->A[i] = *p++;
			}
		}

		br->Close();
		fs->Close();


		delete [] buf;
		return npts;
	}


	int GraphDoc::ReadTextDataFile(String^ strFileName, int nt)
	{
		int i,j,k;
		int npts;

        StreamReader^ sr = gcnew StreamReader(strFileName);
		String^ s;
		String^ t;

		array<Char>^ chars ={' '};
		array<String^>^ split;


		// count the number of data items
		npts = 0;

		while(sr->ReadLine()!=nullptr){
			npts++;		
		}

		// rewind and read the data items

		if(npts==0){
			return 0;
		}

		sr->BaseStream->Position = 0;

		pGraph->ntraces = nt;
		pGraph->nwaves = nt+1;
		pGraph->npts = npts;

		pGraph->MakeWaves();		

		for(i=0;i<pGraph->npts;i++){
			s = sr->ReadLine();
			split = s->Split(chars);
			j=0;
			for(k=0;k<split->Length;k++){
				t=split[k];
				if(t->Length > 0){
					pGraph->W[j]->A[i] = System::Convert::ToDouble(t);
					j++;
				}
			}
		}

		sr->Close();

		return npts;
	}


	int GraphDoc::OpenFile(String^ sFileName, int nTraces)
	{
		int nt,npts;


		if(sFileName==nullptr){
			OpenFileDialog^ openFileDialog1 = gcnew OpenFileDialog;
			openFileDialog1->Filter = "dat files (*.dat)|*.dat|All files (*.*)|*.*";
			openFileDialog1->FilterIndex = 2;
			openFileDialog1->RestoreDirectory = true;

			if ( openFileDialog1->ShowDialog() != System::Windows::Forms::DialogResult::OK ){
				return 0;
			}

			sFileName = openFileDialog1->FileName;
		}

		if(sFileName->Contains(".dat")){
			if(nTraces==0){

				nt = CountTraceNumberB(sFileName);
			}else{
				nt=nTraces;
			}
			npts = ReadBinaryDataFile(sFileName, nt);
		}else if(sFileName->Contains(".txt")){
			if(nTraces==0){
				nt = CountTraceNumberT(sFileName);
			}else{
				nt = nTraces;
			}
			npts = ReadTextDataFile(sFileName, nt);
		}

		if(npts<=0)return 0;
		
		pGraph->XAxisNo = 0;
		pGraph->FindMinMax();

		pGraph->XOrigin = pGraph->W[0]->aMin;
		pGraph->XDelta  = (pGraph->W[0]->aMax - pGraph->W[0]->aMin)/(pGraph->npts-1);

		pGraph->XPStart = 0;
		pGraph->XMin = pGraph->W[0]->aMin;
		pGraph->XMax = pGraph->W[0]->aMax;

		return npts;
	}
} // end of namespace

