#pragma once
#include "stdafx.h"

namespace gc {

	GraphView::GraphView(void)
	{
		InitializeComponent();

		pGraph = nullptr;
		penDownFlag = false;
		lineDrawn = false;
		m_bMultiColor = false;
	}

	GraphView::~GraphView()
	{
	}

	void GraphView::InitializeComponent(void)
	{

		AutoSize = true;
		AutoSizeMode = System::Windows::Forms::AutoSizeMode::GrowAndShrink;
		BackColor = System::Drawing::SystemColors::Control;
		Dock = System::Windows::Forms::DockStyle::Fill;
		Name = L"GraphView";
		TabIndex = 1;
		
		MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphView::GraphView_MouseDown);
		MouseUp   += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphView::GraphView_MouseUp);
		MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphView::GraphView_MouseMove);
		Paint += gcnew System::Windows::Forms::PaintEventHandler(this, &GraphView::GraphView_Paint);

	}

	void GraphView::ChangeMode()
	{
		
		ModeDialog^ md = gcnew ModeDialog(pGraph);
		md->ShowDialog();
		if(md->ResultOK==false)return;

		pGraph->XOrigin = pGraph->W[pGraph->XAxisNo]->aMin;
		pGraph->XDelta  = (pGraph->W[pGraph->XAxisNo]->aMax - pGraph->W[pGraph->XAxisNo]->aMin)/(pGraph->npts-1);

		if(pGraph->XAxisNo==0){
			pGraph->XMin = pGraph->W[pGraph->XAxisNo]->A[pGraph->XPStart];
			pGraph->XMax = pGraph->W[pGraph->XAxisNo]->A[pGraph->XPStart + pGraph->Points - 1];
		}else{
			pGraph->XPStart = 0;
			pGraph->Points = pGraph->npts;
			pGraph->XMin = pGraph->W[pGraph->XAxisNo]->aMin;
			pGraph->XMax = pGraph->W[pGraph->XAxisNo]->aMax;
		}

		return;
	}

	void GraphView::ChangeColorMode()
	{
		m_bMultiColor = (m_bMultiColor) ? false : true ;
	}

///////////////////////////////////////////////
//
//     DRAWING PANEL
//


	void GraphView::GraphView_Paint(System::Object^ sender, System::Windows::Forms::PaintEventArgs^  e) 
	{
		int i, n;
		int TicLength;
		int cX, cY;
		double TicUnit;
		double aX, aY;
		double CmToPixels;
		char t[20];

		String^ s;
		System::Drawing::PointF strPointF;

		if( pGraph==nullptr )return;
		if( pGraph->npts <= 0 ) return;

		System::Drawing::Font^ font = gcnew System::Drawing::Font("Arial", 10);
		SolidBrush^ brush = gcnew SolidBrush(System::Drawing::Color::White);
		Pen^ pen = gcnew Pen(System::Drawing::Color::Gray);

		Graphics^ g = e->Graphics;

		Rectangle r = this->ClientRectangle;
		g->FillRectangle(brush, r);

		CmToPixels = g->DpiX/2.54;
		

		rXMin = pGraph->XMin;
		rXMax = pGraph->XMax;

		rYMin = pGraph->YMin;
		rYMax = pGraph->YMax;

		if(pGraph->XMin == pGraph->XMax)pGraph->XMax = pGraph->XMin + 1.0;
		if(rYMin == rYMax) rYMax = rYMin + 1.0;

// margin: 2.0 cm for left and bottom, 1.0 cm for right and top  
		cXOffset = (int)(CmToPixels * 2.0);
		cYOffset = (int)(r.Bottom-CmToPixels * 2.0);
	
		cXScaleFactor = (r.Right-CmToPixels* 3.0) /(pGraph->XMax-pGraph->XMin); 
		cYScaleFactor = (r.Bottom-CmToPixels* 3.0) /(rYMax-rYMin);
		TicLength = (int)(0.15 * CmToPixels);

// X-Axis 
		TicUnit = AutoRange(pGraph->XMin, pGraph->XMax);
		n = (int)( -System::Math::Log10(TicUnit)+1 );
		if( n < 0 ) n = 0;	

		cX = cXScale(pGraph->XMax);
		g->DrawLine(pen, Point(cXOffset,cYOffset),Point(cX,cYOffset));

		i= (int)(rXMin / TicUnit);
		if( i*TicUnit < rXMin ) i++;
		while(1){
			aX = i*TicUnit;
			if(aX > pGraph->XMax ) break;
			sprintf(t,"%.*f",n,aX);
			s = gcnew String(t);

			cX = cXScale(aX);

			strPointF = (g->MeasureString(s,font)).ToPointF();
			strPointF.X = cX - strPointF.X/2;
			strPointF.Y = cYOffset + strPointF.Y/2;
			g->DrawString(s, font, System::Drawing::Brushes::Blue, strPointF);

			g->DrawLine(pen, Point(cX,cYOffset), Point(cX,cYOffset+TicLength));
			i++;
		}

// Y-Axis 
		g->DrawLine(pen,Point(cXOffset,cYOffset), Point(cXOffset,cYScale(rYMax)));

		TicUnit = AutoRange(rYMin,rYMax);
		n = (int)( -System::Math::Log10(TicUnit)) +1;
		if ( n< 0 ) n= 0;
		i= (int)(rYMin / TicUnit);
		if(i*TicUnit < rYMin)i++;

		while(1){
			aY = i*TicUnit;
			if(aY > rYMax)break;

			sprintf(t,"%.*f",n,aY);
			s = gcnew String(t);
			strPointF = (g->MeasureString(s,font)).ToPointF();

			cY = cYScale(aY);

			strPointF.X = cXOffset - strPointF.X - 2*TicLength;
			strPointF.Y = cY - strPointF.Y/2;

			g->DrawString(s, font, System::Drawing::Brushes::Blue, strPointF);
			g->DrawLine(pen, Point(cXOffset-TicLength,cY), Point(cXOffset,cY));
			i++;
		}

		delete pen;
		delete font;
		delete brush;

		DrawTrace(g);
	}


// //////////////////// utilities /////////////////////
//
//   scaling routines for display 
//


	int GraphView::cXScale( double x )
	{
		return  cXOffset + (int)(cXScaleFactor*(x-rXMin)+0.5) ;
	}

	int GraphView::cYScale( double y)
	{
		return  cYOffset - (int)(cYScaleFactor*(y-rYMin)+0.5) ;
	}

	double GraphView::cXScaleBack( int ix)
	{

		return  rXMin + (ix-cXOffset)/cXScaleFactor;
	}

	int GraphView::cXScaleBackInt(int ix)
	{
		double xt;
		int ixt;

		xt = cXScaleBack(ix);  
		ixt = (int)((xt - pGraph->XOrigin)/(pGraph->XDelta));

		if(ixt<0)ixt = 0;
		if(ixt>pGraph->npts)ixt =pGraph->npts-1;
	
		if(pGraph->W[pGraph->XAxisNo]->A[ixt]<xt){
			while(pGraph->W[pGraph->XAxisNo]->A[ixt]<xt && ixt < pGraph->npts-1){
				ixt++;
			}
		}

		if(pGraph->W[pGraph->XAxisNo]->A[ixt]>xt){
			while(pGraph->W[pGraph->XAxisNo]->A[ixt]<xt && ixt>0){
				ixt--;
			}
		}
	
		if(ixt<0)ixt = 0;
		if(ixt>pGraph->npts)ixt =pGraph->npts-1;
	
		return ixt;
	}

	double GraphView::cYScaleBack(int iy)
	{
		return  rYMin + (cYOffset -iy)/cYScaleFactor;
	}


// =====  Tic routines ===== 

	double GraphView::AutoRange(double ZMin,double ZMax)
	{
		int nTic = 3;
		double ZDelta, ZTic, Zn;

		ZDelta = ZMax-ZMin;
		Zn = (int)System::Math::Log10(ZDelta);
		ZTic = System::Math::Pow(10.0, Zn);
		while(1){
			if ( (ZDelta/ZTic) >= nTic)break;

			ZTic = 0.5 *  ZTic;
			if((ZDelta/ZTic)>= nTic)break;

			ZTic = 0.4 *  ZTic;
			if((ZDelta/ZTic)>=nTic)break;

			ZTic = 0.5 *  ZTic;
			if((ZDelta/ZTic)>=nTic)break;
		}
		return ZTic;
	}



//===== Drawing Graph ===== 
//
//     Draw a graph using information stored in aGraph


	void GraphView::DrawTrace(Graphics^ g)
	{
		int NewX, NewY;
		int i,j,k;
		int OldX = -1000;
		int OldY = -1000;
		int nPen;
		double ax,ay;

		array<Point>^ d;

		Pen^ pen;

		if(m_bMultiColor){
			nPen=8;
		}else{
			nPen=1;
		}

		array<Pen^>^ pens = gcnew array<Pen^>(nPen); 

		for(i=0;i<nPen;i++){
			pens[i] = gcnew Pen(Color::Red);
		}

		if(m_bMultiColor){
			pens[1]->Color = Color::Green;
			pens[2]->Color = Color::Blue;
			pens[3]->Color = Color::Orange;
			pens[4]->Color = Color::DarkBlue;
			pens[5]->Color = Color::Yellow;
			pens[6]->Color = Color::Purple;
			pens[7]->Color = Color::Gray;
		}

		array<double>^ px;
		array<double>^ py;


		px = pGraph->W[pGraph->XAxisNo]->A;

		for(i = 1 ; i < pGraph->nwaves ; i++){

			pen = (m_bMultiColor) ? pens[(i-1)%8] : pens[0];
			


			if(pGraph->W[i]->visible && i!=pGraph->XAxisNo){
				d =  gcnew array<Point>(1024*16); 
				k = 0;

				py = pGraph->W[i]->A;
				for(j = pGraph->XPStart ; j < pGraph->XPStart +pGraph->Points ; j++){

					ax = px[j];
					if(ax>rXMax)ax=rXMax;else if(ax<rXMin)ax=rXMin;
					NewX = cXScale(ax);

					ay=py[j];
					if(ay > rYMax)ay = rYMax; else if(ay < rYMin)ay = rYMin;
					NewY = cYScale(ay);

					if((NewX != OldX) || (NewY != OldY )){
						d[k].X =  OldX = NewX ;
						d[k++].Y =  OldY = NewY ;
					}
					if(k >= 1024*16){
						g->DrawLines(pen,d);
						k=0;
						d[k].X = NewX;
						d[k++].Y = NewY;
					}
				}	
				Array::Resize(d,k);

				g->DrawLines(pen, d);
				delete d;
			}
		}
		for(i=0;i<nPen;i++){
			delete pens[i];
		}
		
		delete [] pens;
	}


/////////////////////////////////////////////////////////////////////////////
// CGView message handlers
//

	void GraphView::GraphView_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e)
	{
		penDownFlag = true;
		originPoint = e->Location;

		Control^ control = dynamic_cast<Control^>(sender);
		startPoint = control->PointToScreen( Point(e->X,e->Y) );
	}

	void GraphView::GraphView_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e)
	{

		if(pGraph==nullptr)return;
		if(penDownFlag==false)return;
		ControlPaint::DrawReversibleFrame( theRectangle, this->BackColor, FrameStyle::Dashed );

		theRectangle = Rectangle(0,0,0,0);

		lineDrawn = false;
		penDownFlag = false;
		endPoint = e->Location;

		if(e->Button == System::Windows::Forms::MouseButtons::Left){  // Left button, X-axis

			int X0 = originPoint.X;
			int X1 = endPoint.X;

			if(X0 == X1){ return; }
			if(X0 > X1){ int tmp=X0; X0=X1, X1=tmp;}

			pGraph->XMin = cXScaleBack(X0);
			pGraph->XMax = cXScaleBack(X1);

			pGraph->findXRange(pGraph->XMin, pGraph->XMax);

/*			if(pGraph->XAxisNo==0){
				X0 = cXScaleBackInt(X0);
				X1 = cXScaleBackInt(X1);
				if(X1-X0 < 1)return;
				pGraph->XPStart = X0;
				pGraph->Points = X1 - X0 + 1;
			}
*/





		}else if (e->Button == System::Windows::Forms::MouseButtons::Right){

			int Y0 = originPoint.Y;
			int Y1 = endPoint.Y;

			if(Y0 == Y1){ return;}
			if(Y0 > Y1){ int tmp=Y0; Y0=Y1, Y1=tmp;}

			pGraph->YMax = cYScaleBack(Y0);
			pGraph->YMin = cYScaleBack(Y1);
		}
		Invalidate();
		
	}

	void GraphView::GraphView_MouseMove(System::Object^ sender, System::Windows::Forms::MouseEventArgs^ e)
	{
		if(pGraph ==nullptr)return;
		if(penDownFlag == false)return;

		ControlPaint::DrawReversibleFrame( theRectangle, this->BackColor, FrameStyle::Dashed );
        
		endPoint = e->Location;

		theRectangle = Rectangle(startPoint.X, startPoint.Y,
							endPoint.X - originPoint.X, endPoint.Y - originPoint.Y);
         
		ControlPaint::DrawReversibleFrame( theRectangle, this->BackColor, FrameStyle::Dashed );
	}

	void GraphView::FullWaves() 
	{

/*
		pGraph->XPStart  = 0;
		pGraph->Points  = pGraph->npts;

		pGraph->XOrigin = pGraph->W[pGraph->XAxisNo]->aMin;
		pGraph->XDelta  = (pGraph->W[pGraph->XAxisNo]->aMax - pGraph->W[pGraph->XAxisNo]->aMin)/(pGraph->npts-1);

		if(pGraph->XAxisNo==0){
			pGraph->XMin = pGraph->W[pGraph->XAxisNo]->A[pGraph->XPStart];
			pGraph->XMax = pGraph->W[pGraph->XAxisNo]->A[pGraph->XPStart + pGraph->Points - 1];
		}else{
			pGraph->XMin = pGraph->W[pGraph->XAxisNo]->aMin;
			pGraph->XMax = pGraph->W[pGraph->XAxisNo]->aMax;
		}
*/
		pGraph->FullGraph();

		Invalidate();
	}


}